'''
Copyright (C) 2024 1P2D
https://1p2d.com
me@1p2d.com

Created by 1P2D - Felix Pierron
'''

import bpy
from bpy.types import Operator


def get_last_node_before_outputs(node_tree):
    """Get the last node before Composite/Viewer nodes"""
    composite = node_tree.nodes.get('Composite')
    if not composite:
        return None
    
    for link in node_tree.links:
        if link.to_node == composite:
            return link.from_node
    return None

def ensure_nodes(node_tree):
    """Ensure proper node setup with input and output nodes"""
    # Get or create Render Layers node (input)
    render_layers = node_tree.nodes.get('Render Layers')
    if not render_layers:
        render_layers = node_tree.nodes.new(type='CompositorNodeRLayers')
        render_layers.location = (0, 0)

    # Get or create output nodes
    composite_node = node_tree.nodes.get('Composite')
    if not composite_node:
        composite_node = node_tree.nodes.new(type='CompositorNodeComposite')
        composite_node.location = (render_layers.location.x + 300, render_layers.location.y)

    viewer_node = node_tree.nodes.get('Viewer')
    if not viewer_node:
        viewer_node = node_tree.nodes.new(type='CompositorNodeViewer')
        viewer_node.location = (render_layers.location.x + 300, render_layers.location.y - 200)

    # Find all effect nodes
    background_over = node_tree.nodes.get('Background Alpha Over')
    stroke_over = node_tree.nodes.get('Stroke Alpha Over')
    glare_node = node_tree.nodes.get('Glare')

    # Start with render layers as the initial input
    current_output = render_layers.outputs['Image']

    # Chain nodes in the correct order: Background -> Stroke -> Glare
    if background_over:
        # Connect background color and image
        bg_color = node_tree.nodes.get('Background Color')
        if bg_color:
            node_tree.links.new(bg_color.outputs[0], background_over.inputs[1])
        node_tree.links.new(current_output, background_over.inputs[2])
        current_output = background_over.outputs[0]

    if stroke_over:
        # Connect the previous output to the stroke's image input
        node_tree.links.new(current_output, stroke_over.inputs[2])
        current_output = stroke_over.outputs[0]

    if glare_node:
        # Connect the previous output to glare
        node_tree.links.new(current_output, glare_node.inputs[0])
        current_output = glare_node.outputs[0]

    # Remove existing output connections
    for link in node_tree.links:
        if link.to_node in {composite_node, viewer_node}:
            node_tree.links.remove(link)

    # Connect final output to composite and viewer
    node_tree.links.new(current_output, composite_node.inputs[0])
    node_tree.links.new(current_output, viewer_node.inputs[0])

    # Update node positions based on chain order
    x_offset = render_layers.location.x
    spacing = 300
    
    if background_over:
        x_offset += spacing
        background_over.location.x = x_offset
        
    if stroke_over:
        x_offset += spacing
        stroke_over.location.x = x_offset
        
    if glare_node:
        x_offset += spacing
        glare_node.location.x = x_offset
        
    # Position output nodes after the last effect
    composite_node.location.x = x_offset + spacing
    viewer_node.location.x = x_offset + spacing



class ToggleTransparency(Operator):
    """Toggle Transparency"""
    bl_idname = "shading_plus.toggle_transparency"
    bl_label = "Toggle Transparency"
    bl_description = "Toggle render transparency (transparent background)"

    def execute(self, context):
        context.scene.render.film_transparent = not context.scene.render.film_transparent
        self.report({'INFO'}, "Transparency " + ("enabled" if context.scene.render.film_transparent else "disabled"))
        return {'FINISHED'}



class ToggleCompositor(Operator):
    bl_idname = "shading_plus.toggle_compositor"
    bl_label = "Toggle Compositor"
    bl_description = "Toggle the compositor's use of nodes on or off"

    def execute(self, context):
        scene = context.scene
        node_tree = scene.node_tree
        compositor_active = scene.use_nodes if scene else False

        if compositor_active:
            scene.use_nodes = False
            self.report({'INFO'}, "Compositor nodes turned off")
        else:
            scene.use_nodes = True
            self.report({'INFO'}, "Compositor nodes turned on")

        ensure_nodes(node_tree)
        return {'FINISHED'}






class AddBackground(Operator):
    """Add/Remove Background"""
    bl_idname = "shading_plus.add_background"
    bl_label = "Set Background Color"
    bl_description = "Add a background color to the scene (Alt+Click to remove)"
    bl_options = {'REGISTER', 'UNDO'}

    color: bpy.props.FloatVectorProperty(
        name="Color",
        subtype='COLOR',
        size=4,
        min=0.0,
        max=1.0,
        default=(0.5, 0.5, 0.5, 1.0),
        options={'SKIP_SAVE'}
    )

    def execute(self, context):
        scene = context.scene
        scene.use_nodes = True
        node_tree = scene.node_tree

        # Get or create nodes
        render_layers = node_tree.nodes.get('Render Layers')
        if not render_layers:
            render_layers = node_tree.nodes.new(type='CompositorNodeRLayers')
            render_layers.location = 0, 0

        rgb_node = node_tree.nodes.get('Background Color')
        if not rgb_node:
            rgb_node = node_tree.nodes.new(type='CompositorNodeRGB')
            rgb_node.name = 'Background Color'
            rgb_node.location = (-200, 0)

        alpha_over = node_tree.nodes.get('Background Alpha Over')
        if not alpha_over:
            alpha_over = node_tree.nodes.new(type='CompositorNodeAlphaOver')
            alpha_over.name = 'Background Alpha Over'
            alpha_over.label = "add background"
            alpha_over.location = (200, 0)

        rgb_node.outputs[0].default_value = self.color

        # Create links
        node_tree.links.new(rgb_node.outputs[0], alpha_over.inputs[1])
        node_tree.links.new(render_layers.outputs['Image'], alpha_over.inputs[2])

        ensure_nodes(node_tree)
        return {'FINISHED'}

    def modal(self, context, event):
        if event.type == 'LEFTMOUSE':
            return {'FINISHED'}
        elif event.type == 'MOUSEMOVE':
            return self.execute(context)
        elif event.type == 'ESC':
            return {'CANCELLED'}
        return {'RUNNING_MODAL'}

    def invoke(self, context, event):
        if event.alt:
            self.remove_background(context)
            return {'FINISHED'}
            
        self.execute(context)
        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}

    def remove_background(self, context):
        node_tree = context.scene.node_tree
        if not node_tree:
            return
            
        # Get nodes to remove
        alpha_over = node_tree.nodes.get('Background Alpha Over')
        rgb_node = node_tree.nodes.get('Background Color')
        
        if not alpha_over:
            return

        # Find connections
        input_node = None
        output_node = None
        for link in node_tree.links:
            if link.to_node == alpha_over and link.to_socket == alpha_over.inputs[2]:
                input_node = link.from_node
            if link.from_node == alpha_over:
                output_node = link.to_node

        # Remove nodes
        if rgb_node:
            node_tree.nodes.remove(rgb_node)
        if alpha_over:
            node_tree.nodes.remove(alpha_over)

        # Reconnect
        if input_node and output_node:
            node_tree.links.new(input_node.outputs[0], output_node.inputs[0])

        ensure_nodes(node_tree)

class AddGlare(Operator):
    """Add/Remove Glare"""
    bl_idname = "shading_plus.add_glare"
    bl_label = "Add Glare"
    bl_description = "Add a glare effect (Alt+Click to remove)"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        scene = context.scene
        scene.use_nodes = True
        node_tree = scene.node_tree

        # Find the Alpha Over node or last node in chain
        alpha_over = node_tree.nodes.get('Background Alpha Over')
        if not alpha_over:
            previous_node = get_last_node_before_outputs(node_tree)
            if not previous_node:
                previous_node = node_tree.nodes.get('Render Layers')
                if not previous_node:
                    previous_node = node_tree.nodes.new(type='CompositorNodeRLayers')
                    previous_node.location = 0, 0
        else:
            previous_node = alpha_over

        # Get or create Glare node
        glare_node = node_tree.nodes.get('Glare')
        if glare_node:
            # Reset existing glare node settings
            glare_node.glare_type = 'BLOOM'
            glare_node.threshold = 2.9
        else:
            glare_node = node_tree.nodes.new(type='CompositorNodeGlare')
            glare_node.name = 'Glare'
            glare_node.label = "add glare"
            glare_node.location = (previous_node.location.x + 300, previous_node.location.y)
            glare_node.glare_type = 'BLOOM'
            glare_node.threshold = 2.9

        # Connect previous node to Glare
        node_tree.links.new(previous_node.outputs[0], glare_node.inputs[0])
        ensure_nodes(node_tree)
        return {'FINISHED'}

    def invoke(self, context, event):
        if event.alt:
            self.remove_glare(context)
            return {'FINISHED'}
        return self.execute(context)

    def remove_glare(self, context):
        node_tree = context.scene.node_tree
        glare_node = node_tree.nodes.get('Glare')
        
        if glare_node:
            prev_node = None
            next_node = None
            
            for link in node_tree.links:
                if link.to_node == glare_node:
                    prev_node = link.from_node
                if link.from_node == glare_node:
                    next_node = link.to_node
            
            node_tree.nodes.remove(glare_node)
            
            if prev_node and next_node:
                node_tree.links.new(prev_node.outputs[0], next_node.inputs[0])
        
        ensure_nodes(node_tree)

class AddStroke(Operator):
    """Add/Remove Stroke Effect"""
    bl_idname = "shading_plus.add_stroke"
    bl_label = "Add Stroke"
    bl_description = "Add a stroke effect around objects (Alt+Click to remove)"
    bl_options = {'REGISTER', 'UNDO'}

    distance: bpy.props.IntProperty(
        name="Distance",
        description="Stroke thickness",
        default=1,
        min=0,
        max=100
    )

    color: bpy.props.FloatVectorProperty(
        name="Color",
        subtype='COLOR',
        size=4,
        min=0.0,
        max=1.0,
        default=(1.0, 1.0, 1.0, 1.0)
    )

    @classmethod
    def poll(cls, context):
        return context.scene.node_tree is not None

    def remove_nodes(self, context):
        node_tree = context.scene.node_tree
        if not node_tree:
            return

        # First find the frame and alpha over
        stroke_frame = None
        stroke_over = None
        for node in node_tree.nodes:
            if node.type == 'FRAME' and node.label == "add stroke":
                stroke_frame = node
            elif node.name == "Stroke Alpha Over":
                stroke_over = node

        if not stroke_frame or not stroke_over:
            return

        # Find connections
        input_node = None
        output_node = None
        for link in node_tree.links:
            if link.to_node == stroke_over and link.to_socket == stroke_over.inputs[2]:
                input_node = link.from_node
            if link.from_node == stroke_over:
                output_node = link.to_node

        # Remove all nodes in frame
        nodes_to_remove = [n for n in node_tree.nodes if n.parent == stroke_frame]
        for node in nodes_to_remove:
            node_tree.nodes.remove(node)

        # Remove frame and alpha over
        if stroke_frame:
            node_tree.nodes.remove(stroke_frame)
        if stroke_over:
            node_tree.nodes.remove(stroke_over)

        # Reconnect
        if input_node and output_node:
            node_tree.links.new(input_node.outputs[0], output_node.inputs[0])

        ensure_nodes(node_tree)

    def execute(self, context):
        scene = context.scene
        scene.use_nodes = True
        node_tree = scene.node_tree

        render_layers = node_tree.nodes.get('Render Layers')
        if not render_layers:
            render_layers = node_tree.nodes.new(type='CompositorNodeRLayers')
            render_layers.location = 0, 0

        # Get or create frame and nodes
        stroke_frame = None
        dilate_node = None
        rgb_node = None
        
        for node in node_tree.nodes:
            if node.type == 'FRAME' and node.label == "add stroke":
                stroke_frame = node
            elif node.type == 'DILATE_ERODE' and node.parent and node.parent.label == "add stroke":
                dilate_node = node
            elif node.type == 'RGB' and node.parent and node.parent.label == "add stroke":
                rgb_node = node

        if not stroke_frame:
            # Create new setup
            stroke_frame = node_tree.nodes.new(type='NodeFrame')
            stroke_frame.label = "add stroke"
            stroke_frame.use_custom_color = True
            stroke_frame.color = (0.6, 0.6, 0.6)

            reroute = node_tree.nodes.new(type='NodeReroute')
            reroute.location = (render_layers.location.x + 200, render_layers.location.y - 200)
            reroute.parent = stroke_frame

            dilate = node_tree.nodes.new(type='CompositorNodeDilateErode')
            dilate.location = (reroute.location.x + 200, reroute.location.y)
            dilate.parent = stroke_frame
            dilate_node = dilate

            mix = node_tree.nodes.new(type='CompositorNodeMixRGB')
            mix.blend_type = 'MULTIPLY'
            mix.location = (dilate.location.x + 200, dilate.location.y)
            mix.parent = stroke_frame

            rgb = node_tree.nodes.new(type='CompositorNodeRGB')
            rgb.location = (mix.location.x, mix.location.y - 150)
            rgb.parent = stroke_frame
            rgb_node = rgb

            alpha_over = node_tree.nodes.new(type='CompositorNodeAlphaOver')
            alpha_over.name = "Stroke Alpha Over"
            alpha_over.location = (mix.location.x + 200, mix.location.y)

            # Create links
            node_tree.links.new(render_layers.outputs['Alpha'], reroute.inputs[0])
            node_tree.links.new(reroute.outputs[0], dilate.inputs[0])
            node_tree.links.new(dilate.outputs[0], mix.inputs[1])
            node_tree.links.new(rgb.outputs[0], mix.inputs[2])
            node_tree.links.new(mix.outputs[0], alpha_over.inputs[1])
                
            # Only connect alpha to factor if there's no background node
            background_over = node_tree.nodes.get('Background Alpha Over')
            if not background_over:
                node_tree.links.new(reroute.outputs[0], alpha_over.inputs[0])
            else:
                # If there is a background, feed a white value to the factor
                alpha_over.inputs[0].default_value = 1

        # Always update values
        if dilate_node:
            dilate_node.distance = self.distance
        if rgb_node:
            rgb_node.outputs[0].default_value = self.color

        ensure_nodes(node_tree)
        return {'FINISHED'}

    def modal(self, context, event):
        if event.type == 'LEFTMOUSE':
            return {'FINISHED'}
        elif event.type == 'MOUSEMOVE':
            return self.execute(context)
        elif event.type == 'ESC':
            self.remove_nodes(context)
            return {'CANCELLED'}
        return {'RUNNING_MODAL'}

    def invoke(self, context, event):
        if event.alt:
            self.remove_nodes(context)
            return {'FINISHED'}
            
        self.execute(context)
        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}

def ensure_nodes(node_tree):
    """Ensure proper node setup with input and output nodes"""
    # Get or create Render Layers node (input)
    render_layers = node_tree.nodes.get('Render Layers')
    if not render_layers:
        render_layers = node_tree.nodes.new(type='CompositorNodeRLayers')
        render_layers.location = (0, 0)

    # Get or create output nodes
    composite_node = node_tree.nodes.get('Composite')
    if not composite_node:
        composite_node = node_tree.nodes.new(type='CompositorNodeComposite')
        composite_node.location = (render_layers.location.x + 300, render_layers.location.y)

    viewer_node = node_tree.nodes.get('Viewer')
    if not viewer_node:
        viewer_node = node_tree.nodes.new(type='CompositorNodeViewer')
        viewer_node.location = (render_layers.location.x + 300, render_layers.location.y - 200)

    # Find all effect nodes
    background_over = node_tree.nodes.get('Background Alpha Over')
    stroke_over = node_tree.nodes.get('Stroke Alpha Over')
    glare_node = node_tree.nodes.get('Glare')

    # Start with render layers as the initial input
    current_output = render_layers.outputs['Image']

    # Chain nodes in the correct order: Background -> Stroke -> Glare
    if background_over:
        # If background exists but is after stroke, move it before
        if stroke_over and background_over.location.x > stroke_over.location.x:
            # Move background nodes before stroke
            bg_color = node_tree.nodes.get('Background Color')
            if bg_color:
                bg_color.location.x = stroke_over.location.x - 400
            background_over.location.x = stroke_over.location.x - 300
            
        # Connect background color and image
        bg_color = node_tree.nodes.get('Background Color')
        if bg_color:
            node_tree.links.new(bg_color.outputs[0], background_over.inputs[1])
        node_tree.links.new(current_output, background_over.inputs[2])
        # Connect alpha
        node_tree.links.new(render_layers.outputs['Alpha'], background_over.inputs[0])
        current_output = background_over.outputs[0]

    if stroke_over:
        # Connect the previous output to the stroke's image input
        node_tree.links.new(current_output, stroke_over.inputs[2])
        current_output = stroke_over.outputs[0]

    if glare_node:
        # Connect the previous output to glare
        node_tree.links.new(current_output, glare_node.inputs[0])
        current_output = glare_node.outputs[0]

    # Remove existing output connections
    for link in node_tree.links:
        if link.to_node in {composite_node, viewer_node}:
            node_tree.links.remove(link)

    # Connect final output to composite and viewer
    node_tree.links.new(current_output, composite_node.inputs[0])
    node_tree.links.new(current_output, viewer_node.inputs[0])

    # Update node positions based on chain order
    x_offset = render_layers.location.x
    spacing = 300
    
    if background_over:
        x_offset += spacing
        background_over.location.x = x_offset
        bg_color = node_tree.nodes.get('Background Color')
        if bg_color:
            bg_color.location.x = x_offset - 200
        
    if stroke_over:
        x_offset += spacing
        stroke_over.location.x = x_offset
        
    if glare_node:
        x_offset += spacing
        glare_node.location.x = x_offset
        
    # Position output nodes after the last effect
    composite_node.location.x = x_offset + spacing
    viewer_node.location.x = x_offset + spacing
# REGISTER/UNREGISTER
classes = (
    ToggleCompositor,
    ToggleTransparency,  # Add this line
    AddBackground,
    AddGlare,
    AddStroke,
)


def register():
    for cls in classes:
        bpy.utils.register_class(cls)

def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)