import bpy
import os
from bpy.types import Menu

# Import the custom_icons from icons.py
from . import icons

class PIE_MT_1P2D_shading_plus(Menu):
    bl_label = "Shading +"
    bl_idname = "PIE_MT_1P2D_shading_plus"

    def draw(self, context):
        layout = self.layout
        pie = layout.menu_pie()

        scene = context.scene  # Access the current scene

        # Access addon preferences
        addon_name = __package__
        addon_prefs = context.preferences.addons[addon_name].preferences
        show_preset_names = addon_prefs.show_preset_names

        space = context.space_data

        preset_icons = {
            'ALIAS1': 'shadesky',
            'ALIAS2': 'showroom',
            'ALIAS3': 'photo_horizon',
            'ALIAS4': 'double_horizon',
            'RED1': 'redgloss_better',
            'RED2': 'redgloss_oldblender',
            'FUEL': 'fuel_shaded',
            'CLOWN': 'clown',
        }

        # Updated mapping from preset IDs to required matcap names (as used in operators.py)
        preset_required_matcaps = {
            'ALIAS1': 'Alias - Shade Sky.exr',
            'ALIAS2': 'Alias - Showroom.exr',
            'ALIAS3': 'Alias - Photo Horizon.exr',
            'ALIAS4': 'Alias - Double Horizon.exr',
            'RED1': 'Misc - Red Gloss Better.exr',
            'RED2': 'Misc - Red Gloss Old Blender.exr',
            'FUEL': 'Misc - Fuel Shaded.exr',
        }

        # Collect the 'name' property of installed matcaps
        installed_matcap_names = set()
        for sl in bpy.context.preferences.studio_lights:
            if sl.type == 'MATCAP':
                installed_matcap_names.add(sl.name)

        # For debugging: print the list of installed matcap names
        # Uncomment the following lines to print installed matcaps
        # print("Installed Matcaps:")
        # for name in installed_matcap_names:
        #     print(f"'{name}'")

        # 1. West (Left)
        wire_op = pie.operator("wm.context_set_enum", text="Wireframe", icon='SHADING_WIRE')
        wire_op.data_path = "space_data.shading.type"
        wire_op.value = 'WIREFRAME'





        # 2. East (Right)
        col = pie.column(align=True)
        box = col.box()
        box_col = box.column(align=True)

        # Line 1: Solid Shading Operator with dynamic icon based on selected preset
        row = box_col.row(align=True)
        row.scale_x = 1
        row.scale_y = 1.3

        # Access the shading preset property and determine the icon to use
        shading_props = scene.shading_plus
        selected_preset = shading_props.shading_preset

        # Determine the icon based on the selected preset
        if selected_preset == 'DEFAULT':
            solid_icon = 'BLENDER'
            solid_icon_value = None
        elif selected_preset == 'CELL':
            solid_icon = 'MONKEY'
            solid_icon_value = None
        elif selected_preset == 'USER1':
            solid_icon = 'USER'
            solid_icon_value = None
        else:
            # Use custom icons for other presets
            custom_icon_name = preset_icons.get(selected_preset, None)
            if custom_icon_name and icons.custom_icons.get(custom_icon_name):
                # Retrieve custom icon's icon_id if available
                solid_icon_value = icons.custom_icons[custom_icon_name].icon_id
            else:
                # Fallback to default 'SHADING_SOLID' if no icon found
                solid_icon = 'SHADING_SOLID'
                solid_icon_value = None

        # Create the Solid Shading operator with the determined icon
        if selected_preset in ['DEFAULT', 'CELL', 'USER1']:
            # Use the built-in icon for Blender's default, cell, and user presets
            solid_op = row.operator(
                "wm.context_set_enum",
                text="Solid",
                icon=solid_icon,
                depress=(space.shading.type == 'SOLID'),
                emboss=True
            )
        else:
            # Use custom icon if available; otherwise fallback to default
            solid_op = row.operator(
                "wm.context_set_enum",
                text="Solid",
                icon_value=solid_icon_value if 'solid_icon_value' in locals() else bpy.types.UILayout.icon('SHADING_SOLID'),
                depress=(space.shading.type == 'SOLID'),
                emboss=True
            )

        solid_op.data_path = "space_data.shading.type"
        solid_op.value = 'SOLID'

        # Check if the CELL preset is selected, and if so, add the extra button
        if selected_preset == 'CELL':
            # Add "Apply Viewport Colors" button next to the Solid Shading button
            row.operator(
                "shading_plus.apply_viewport_colors",
                text="Unify colors",
                icon="COLOR"
            )

        # Line 2: Shading Preset Toggle Buttons
        row = box.row(align=True)
        row.alignment = 'CENTER'  # Center align the presets

        # Get the items of the EnumProperty
        preset_items = shading_props.bl_rna.properties['shading_preset'].enum_items

        for item in preset_items:
            preset_id = item.identifier
            preset_name = item.name if show_preset_names else ""

            # Determine if the preset requires a custom matcap
            if preset_id in preset_required_matcaps:
                required_matcap_name = preset_required_matcaps[preset_id]
                matcap_installed = required_matcap_name in installed_matcap_names
            else:
                matcap_installed = True  # Preset doesn't require a custom matcap

            if preset_id == 'DEFAULT':
                icon = 'BLENDER'
                icon_value = None
            else:
                custom_icon_name = preset_icons.get(preset_id, None)
                if custom_icon_name and icons.custom_icons.get(custom_icon_name):
                    icon_value = icons.custom_icons[custom_icon_name].icon_id
                else:
                    icon_value = None

            # Create a sub-layout for the operator to control the enabled state
            sub = row.row(align=True)
            sub.enabled = matcap_installed  # Disable if matcap not installed

            if preset_id == 'DEFAULT':
                op = sub.operator(
                    'wm.context_set_enum',
                    text=preset_name,
                    icon='BLENDER',
                    depress=(shading_props.shading_preset == preset_id),
                    emboss=True
                )
            elif preset_id == 'CELL':
                op = sub.operator(
                    'wm.context_set_enum',
                    text=preset_name,
                    icon='MONKEY',
                    depress=(shading_props.shading_preset == preset_id),
                    emboss=True
                )
            elif preset_id == 'USER1':
                op = sub.operator(
                    'wm.context_set_enum',
                    text=preset_name,
                    icon='USER',
                    depress=(shading_props.shading_preset == preset_id),
                    emboss=True
                )
            else:
                if icon_value:
                    op = sub.operator(
                        'wm.context_set_enum',
                        text=preset_name,
                        icon_value=icon_value,
                        depress=(shading_props.shading_preset == preset_id),
                        emboss=True
                    )
                else:
                    op = sub.operator(
                        'wm.context_set_enum',
                        text=preset_name,
                        icon='QUESTION',
                        depress=(shading_props.shading_preset == preset_id),
                        emboss=True
                    )

            op.data_path = "scene.shading_plus.shading_preset"
            op.value = preset_id





        # 3. South (Bottom)
        # Organize buttons into two lines
        col = pie.column(align=True)
        box = col.box()
        box_col = box.column(align=True)

        # Line 1: Toggle Compositor and Transparency Buttons
        row = box_col.row(align=True)
        row.scale_x = 1
        row.scale_y = 1.3

        # Toggle Compositor Button
        compositor_active = scene.use_nodes if scene else False
        toggle_comp_op = row.operator(
            "shading_plus.toggle_compositor",
            text="Compositor",
            icon='CHECKBOX_HLT' if compositor_active else 'CHECKBOX_DEHLT',
            depress=compositor_active
        )

        # Toggle Transparency Button
        transparency_active = scene.render.film_transparent
        toggle_trans_op = row.operator(
            "shading_plus.toggle_transparency",
            text="Transparent",
            icon='CHECKBOX_HLT' if transparency_active else 'CHECKBOX_DEHLT',
            depress=transparency_active
        )

        # Add spacing between lines
        box_col.separator(factor=0.5)

        # Line 2: Add Background and Add Glare Buttons
        row = box_col.row(align=True)
        row.alignment = 'CENTER'  # Center align the buttons


        # Add Background Button
        #add_bg_op = row.operator(
            #"shading_plus.add_background",
            #text="",
            #icon='COLOR'
        #)

        # Add Glare Button
        #add_glare_op = row.operator(
            #"shading_plus.add_glare",
            #text="",
            #icon='GEOMETRY_NODES'
        #)

        # Add Stroke Button
        #add_stroke_op = row.operator(
            #"shading_plus.add_stroke",
            #text="",
            #icon='GREASEPENCIL'
        #)






        # 4. North (Top)
        pie.operator("shading_plus.set_cycles_rendered_view", text="Cycles", icon='SHADING_RENDERED')

        # 5. Northwest (Top-Left)
        pie.operator("shading_plus.set_eevee_rendered_view", text="EEVEE", icon='SHADING_RENDERED')

        # 6. Northeast (Top-Right)
        pie.operator("shading_plus.set_workbench_rendered_view", text="Workbench", icon='SHADING_RENDERED')

        # 7. Southwest (Bottom-Left)
        pie.operator("shading_plus.set_material_preview", text="Material", icon='MATERIAL')






        # Southeast Position
        space = context.space_data

        # Determine if buttons should be disabled
        current_shading_type = space.shading.type
        disable_buttons = current_shading_type in {'WIREFRAME', 'SOLID'}

        col = pie.column()
        box_hdri = col.box()
        row_hdri_toggle = box_hdri.row(align=True)

        row_hdri_toggle.scale_y = 1.2  # Adjust button height if needed

        # Set 'enabled' on the row to disable both buttons
        row_hdri_toggle.enabled = not disable_buttons

        if not disable_buttons:
            if current_shading_type == 'RENDERED':
                use_scene_world = space.shading.use_scene_world_render
            else:
                use_scene_world = space.shading.use_scene_world

            current_option = 'WORLD' if use_scene_world else 'BUILTIN'
        else:
            current_option = None  # Neither button is depressed when disabled

        op_builtin = row_hdri_toggle.operator(
            "shading_plus.set_hdri_option",
            text="Studiolight",
            icon='BLENDER',
            depress=(current_option == 'BUILTIN') if current_option else False
        )
        op_builtin.option = 'BUILTIN'

        op_scene = row_hdri_toggle.operator(
            "shading_plus.set_hdri_option",
            text="World",
            icon='WORLD',
            depress=(current_option == 'WORLD') if current_option else False
        )
        op_scene.option = 'WORLD'

        # Draw the built-in HDRI submenu below the toggle buttons
        col_hdri = box_hdri.column(align=True)
        col_hdri.enabled = (current_option == 'BUILTIN')

        # Access the built_in_hdri property
        shading_props = scene.shading_plus

        # Get the items of the EnumProperty
        hdri_items = shading_props.bl_rna.properties['built_in_hdri'].enum_items

        # Mapping of built-in HDRI identifiers to custom icon names
        hdri_icon_mapping = {
            'city.exr': 'hdri_city',
            'courtyard.exr': 'hdri_courtyard',
            'forest.exr': 'hdri_forest',
            'interior.exr': 'hdri_interior',
            'night.exr': 'hdri_night',
            'studio.exr': 'hdri_studio',
            'sunrise.exr': 'hdri_sunrise',
            'sunset.exr': 'hdri_sunset',
        }

        # Create a single row for all HDRI buttons
        row_hdri = col_hdri.row(align=True)
        row_hdri.alignment = 'CENTER'

        for item in hdri_items:
            hdri_id = item.identifier
            hdri_name = item.name

            # Get the corresponding custom icon name
            icon_name = hdri_icon_mapping.get(hdri_id, None)

            if icon_name and icons.custom_icons.get(icon_name):
                # Use the custom icon
                icon_value = icons.custom_icons[icon_name].icon_id
            else:
                # Fallback to Blender's 'QUESTION' icon if custom icon not found
                icon_value = 'QUESTION'

            # Add the operator with custom icon to the same row
            op = row_hdri.operator(
                'wm.context_set_enum',
                text=hdri_name,
                icon_value=icon_value if isinstance(icon_value, int) else 'QUESTION',
                depress=(shading_props.built_in_hdri == hdri_id),
                emboss=True
            )
            op.data_path = "scene.shading_plus.built_in_hdri"
            op.value = hdri_id
# REGISTER/UNREGISTER
# __________________________________________________________________

classes = (
    PIE_MT_1P2D_shading_plus,
)

def register():
    for cls in classes:
        bpy.utils.register_class(cls)

def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)

if __name__ == "__main__":
    register()
